@@ -34,6 +34,7 @@
 
 static gchar* primary_text;
 static gchar* clipboard_text;
+static gchar* synchronized_text;
 static GtkClipboard* primary;
 static GtkClipboard* clipboard;
 static GtkStatusIcon* status_icon;
@@ -41,38 +42,26 @@ static GtkStatusIcon* status_icon;
 static gboolean actions_lock = FALSE;
 
 /* Init preferences structure */
-prefs_t prefs = {DEF_USE_COPY,        DEF_USE_PRIMARY,
+prefs_t prefs = {DEF_USE_COPY,        DEF_USE_PRIMARY,      DEF_SYNCHRONIZE,
                  DEF_SAVE_HISTORY,    DEF_HISTORY_LIMIT,
                  DEF_HYPERLINKS_ONLY, DEF_CONFIRM_CLEAR,
-                 DEF_SINGLE_LINE,     DEF_REVERSE_HISTORY,
-                 DEF_ITEM_LENGTH,     DEF_ELLIPSIZE,
-                 INIT_HISTORY_KEY,    INIT_ACTIONS_KEY,
-                 INIT_MENU_KEY,       DEF_NO_ICON};
+                 DEF_SINGLE_LINE,     DEF_REVERSE_HISTORY,  DEF_ITEM_LENGTH,
+                 DEF_ELLIPSIZE,
+                 INIT_HISTORY_KEY,    INIT_ACTIONS_KEY,     INIT_MENU_KEY,
+                 DEF_NO_ICON};
 
 
-/* Goes through each in history and returns TRUE if item exists in history */
-static gboolean
-item_exists(gchar* text)
-{
-  GSList* element;
-  /* Go through each element compare each */
-  for (element = history; element != NULL; element = element->next)
-  {
-    if (g_strcmp0((gchar*)element->data, text) == 0)
-    {
-      return TRUE;
-    }
-  }
-  return FALSE;
-}
 
 /* Called every CHECK_INTERVAL seconds to check for new items */
 static gboolean
 item_check(gpointer data)
 {
-  /* Process text and restore empty if needed */
+  /* Grab the current primary and clipboard text */
   gchar* primary_temp = gtk_clipboard_wait_for_text(primary);
   gchar* clipboard_temp = gtk_clipboard_wait_for_text(clipboard);
+  
+  /* What follows is an extremely confusing system of tests and crap... */
+  
   /* Check if primary contents were lost */
   if ((primary_temp == NULL) && (primary_text != NULL))
   {
@@ -80,15 +69,36 @@ item_check(gpointer data)
   }
   else
   {
-    /* Get the button state to check if the mouse button is being held */
     GdkModifierType button_state;
     gdk_window_get_pointer(NULL, NULL, NULL, &button_state);
+    /* Proceed if mouse button not being held */
     if ((primary_temp != NULL) && !(button_state & GDK_BUTTON1_MASK))
     {
-      g_free(primary_text);
-      primary_text = g_strdup(primary_temp);
+      /* Check if primary is the same as the last entry */
+      if (g_strcmp0(primary_temp, primary_text) != 0)
+      {
+        /* New primary entry */
+        g_free(primary_text);
+        primary_text = g_strdup(primary_temp);
+        /* Check if primary option is enabled and if there's text to add */
+        if (prefs.use_primary && primary_text)
+        {
+          /* Check contents before adding */
+          if (prefs.hyperlinks_only && is_hyperlink(primary_text))
+          {
+            delete_duplicate(primary_text);
+            append_item(primary_text);
+          }
+          else
+          {
+            delete_duplicate(primary_text);
+            append_item(primary_text);
+          }
+        }
+      }
     }
   }
+  
   /* Check if clipboard contents were lost */
   if ((clipboard_temp == NULL) && (clipboard_text != NULL))
   {
@@ -96,48 +106,51 @@ item_check(gpointer data)
   }
   else
   {
-    g_free(clipboard_text);
-    clipboard_text = g_strdup(clipboard_temp);
+    /* Check if clipboard is the same as the last entry */
+    if (g_strcmp0(clipboard_temp, clipboard_text) != 0)
+    {
+      /* New clipboard entry */
+      g_free(clipboard_text);
+      clipboard_text = g_strdup(clipboard_temp);
+      /* Check if clipboard option is enabled and if there's text to add */
+      if (prefs.use_copy && clipboard_text)
+      {
+        /* Check contents before adding */
+        if (prefs.hyperlinks_only && is_hyperlink(clipboard_text))
+        {
+          delete_duplicate(clipboard_text);
+          append_item(clipboard_text);
+        }
+        else
+        {
+          delete_duplicate(clipboard_text);
+          append_item(clipboard_text);
+        }
+      }
+    }
   }
+  
+  /* Synchronization */
+  if (prefs.synchronize)
+  {
+    if (g_strcmp0(synchronized_text, primary_text) != 0)
+    {
+      g_free(synchronized_text);
+      synchronized_text = g_strdup(primary_text);
+      gtk_clipboard_set_text(clipboard, primary_text, -1);
+    }
+    else if (g_strcmp0(synchronized_text, clipboard_text) != 0)
+    {
+      g_free(synchronized_text);
+      synchronized_text = g_strdup(clipboard_text);
+      gtk_clipboard_set_text(primary, clipboard_text, -1);
+    }
+  }
+  
+  /* Cleanup */
   g_free(primary_temp);
   g_free(clipboard_temp);
-  /* Processing complete... */
   
-  /* Primary check */
-  if (prefs.use_primary)
-  {
-    /* Get the button state to check if the mouse button is being held */
-    GdkModifierType button_state;
-    gdk_window_get_pointer(NULL, NULL, NULL, &button_state);
-    /* Check item */
-    if ((primary_text) && !(button_state & GDK_BUTTON1_MASK))
-    {
-      if (prefs.hyperlinks_only && is_hyperlink(primary_text) && !item_exists(primary_text))
-      {
-        append_item(primary_text);
-      }
-      else if (!prefs.hyperlinks_only && !item_exists(primary_text))
-      {
-        append_item(primary_text);
-      }
-    }
-  }
-  /* Clipboard check */
-  if (prefs.use_copy)
-  {
-    /* Check item */
-    if (clipboard_text)
-    {
-      if (prefs.hyperlinks_only && is_hyperlink(clipboard_text) && !item_exists(clipboard_text))
-      {
-        append_item(clipboard_text);
-      }
-      else if (!prefs.hyperlinks_only && !item_exists(clipboard_text))
-      {
-        append_item(clipboard_text);
-      }
-    }
-  }
   return TRUE;
 }
 
@@ -193,23 +206,6 @@ edit_actions_selected(GtkButton *button, gpointer user_data)
     show_preferences(ACTIONS_TAB);
 }
 
-/* Called when the save button is clicked from the edit window */
-static void
-save_clicked(GtkButton *button, gpointer user_data)
-{
-  /* Get the text buffer, its text and set as clipboard */
-  GtkTextBuffer* text_buffer = gtk_text_view_get_buffer((GtkTextView*)user_data);
-  GtkTextIter start, end;
-  gtk_text_buffer_get_start_iter(text_buffer, &start);
-  gtk_text_buffer_get_end_iter(text_buffer, &end);
-  gchar* new_clipboard_text = gtk_text_buffer_get_text(text_buffer, &start, &end, TRUE);
-  gtk_clipboard_set_text(clipboard, new_clipboard_text, -1);
-  /* Get top-most window widget and destroy it */
-  GtkWidget* window = gtk_widget_get_ancestor((GtkWidget*)user_data, GTK_TYPE_WINDOW);
-  gtk_widget_destroy(window);
-  g_free(new_clipboard_text);
-}
-
 /* Called when Edit is selected from history menu */
 static void
 edit_selected(GtkMenuItem *menu_item, gpointer user_data)
@@ -219,102 +215,48 @@ edit_selected(GtkMenuItem *menu_item, gpointer user_data)
   {
     /* Create clipboard buffer and set its text */
     GtkTextBuffer* clipboard_buffer = gtk_text_buffer_new(NULL);
-    gchar* cb_text = gtk_clipboard_wait_for_text(clipboard);
-    if (cb_text)
-      gtk_text_buffer_set_text(clipboard_buffer, cb_text, -1);
-    g_free(cb_text);
+    gchar* current_clipboard_text = gtk_clipboard_wait_for_text(clipboard);
+    if (current_clipboard_text != NULL)
+    {
+      gtk_text_buffer_set_text(clipboard_buffer, current_clipboard_text, -1);
+    }
     
-    /* Create window */
-    GtkWidget* window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
-    gtk_grab_add(window);
-    gtk_window_set_decorated((GtkWindow*)window, FALSE);
-    gtk_window_set_default_size((GtkWindow*)window, 450, 300);
-    gtk_window_set_keep_above((GtkWindow*)window, TRUE);
-    gtk_window_set_title((GtkWindow*)window, _("Editing Clipboard"));
-    gtk_window_set_icon((GtkWindow*)window, gtk_widget_render_icon(window,
-                                                                   GTK_STOCK_EDIT,
-                                                                   GTK_ICON_SIZE_MENU,
-                                                                   NULL));
+    /* Create the dialog */
+    GtkWidget* dialog = gtk_dialog_new_with_buttons(_("Editing Clipboard"), NULL,
+                                                   (GTK_DIALOG_MODAL   +    GTK_DIALOG_NO_SEPARATOR),
+                                                    GTK_STOCK_CANCEL,       GTK_RESPONSE_REJECT,
+                                                    GTK_STOCK_OK,           GTK_RESPONSE_ACCEPT, NULL);
     
-    /* Add viewport, frame and alignment */
-    GtkWidget* viewport = gtk_viewport_new((GtkAdjustment*)gtk_adjustment_new(0, 0, 0, 0, 0, 0), 
-                                           (GtkAdjustment*)gtk_adjustment_new(0, 0, 0, 0, 0, 0));
+    gtk_window_set_default_size((GtkWindow*)dialog, 450, 300);
+    gtk_window_set_icon((GtkWindow*)dialog, gtk_widget_render_icon(dialog, GTK_STOCK_EDIT, GTK_ICON_SIZE_MENU, NULL));
     
-    /* gtk_viewport_set_shadow_type((GtkViewport*)viewport, GTK_SHADOW_ETCHED_OUT); */
-    gtk_viewport_set_shadow_type((GtkViewport*)viewport, GTK_SHADOW_NONE);
-    gtk_container_add((GtkContainer*)window, viewport);
-    GtkWidget* frame = gtk_frame_new(NULL);
-    gtk_frame_set_shadow_type((GtkFrame*)frame, GTK_SHADOW_NONE);
-    GtkWidget* label = gtk_label_new("Clipboard");
-    gtk_misc_set_padding((GtkMisc*)label, 0, 2);
-    gtk_label_set_markup((GtkLabel*)label, _("<b>Clipboard</b>"));
-    gtk_frame_set_label_widget((GtkFrame*)frame, label);
-    gtk_container_add((GtkContainer*)viewport, frame);
-    GtkWidget* alignment = gtk_alignment_new(0.50, 0.50, 1.0, 1.0);
-    gtk_alignment_set_padding((GtkAlignment*)alignment, 2, 0, 2, 2);
-    gtk_container_add((GtkContainer*)frame, alignment);
-    /* Add vbox for two rows of widgets */
-    GtkWidget* vbox = gtk_vbox_new(FALSE, 0);
-    gtk_container_add((GtkContainer*)alignment, vbox);
-    /* Add inner viewport and textview */
-    GtkWidget* inner_viewport = gtk_viewport_new((GtkAdjustment*)
-                                                 gtk_adjustment_new(0, 0, 0, 0, 0, 0),
-                                                 (GtkAdjustment*)
-                                                 gtk_adjustment_new(0, 0, 0, 0, 0, 0));
-    
-    gtk_viewport_set_shadow_type((GtkViewport*)inner_viewport, GTK_SHADOW_ETCHED_OUT);
-    gtk_box_pack_start(GTK_BOX(vbox), inner_viewport, TRUE, TRUE, 0);
-    GtkWidget* scrolled_window = gtk_scrolled_window_new((GtkAdjustment*)
-                                                         gtk_adjustment_new(0, 0, 0, 0, 0, 0),
-                                                         (GtkAdjustment*)
-                                                         gtk_adjustment_new(0, 0, 0, 0, 0, 0));
+    /* Build the scrolled window with the text view */
+    GtkWidget* scrolled_window = gtk_scrolled_window_new((GtkAdjustment*) gtk_adjustment_new(0, 0, 0, 0, 0, 0),
+                                                         (GtkAdjustment*) gtk_adjustment_new(0, 0, 0, 0, 0, 0));
     
     gtk_scrolled_window_set_policy((GtkScrolledWindow*)scrolled_window,
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
     
-    gtk_container_add((GtkContainer*)inner_viewport, scrolled_window);
+    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), scrolled_window, TRUE, TRUE, 2);
     GtkWidget* text_view = gtk_text_view_new_with_buffer(clipboard_buffer);
     gtk_text_view_set_left_margin((GtkTextView*)text_view, 2);
     gtk_text_view_set_right_margin((GtkTextView*)text_view, 2);
     gtk_container_add((GtkContainer*)scrolled_window, text_view);
-    /* Button box underneath textview */
-    GtkWidget* hbutton_box = gtk_hbutton_box_new();
-    gtk_button_box_set_layout(GTK_BUTTON_BOX(hbutton_box), GTK_BUTTONBOX_END);
-    gtk_box_pack_start(GTK_BOX(vbox), hbutton_box, FALSE, FALSE, 2);
-    GtkWidget* hbox = gtk_hbox_new(FALSE, 2);
-    gtk_box_pack_start(GTK_BOX(hbutton_box), hbox, FALSE, FALSE, 0);
-    /* Save button */
-    GtkWidget* button_save = gtk_button_new();
     
-    gtk_button_set_relief((GtkButton*)button_save, GTK_RELIEF_NONE);
-    gtk_button_set_image((GtkButton*)button_save,
-                         gtk_image_new_from_stock(GTK_STOCK_OK,
-                                                  GTK_ICON_SIZE_MENU));
-    
-    gtk_widget_set_tooltip_text(button_save, _("Save changes"));
-    g_signal_connect((GObject*)button_save, "clicked", (GCallback)save_clicked, text_view);
-    gtk_box_pack_end(GTK_BOX(hbox), button_save, FALSE, FALSE, 0);
-    /* Close button */
-    GtkWidget* button_close = gtk_button_new();
-    gtk_button_set_relief((GtkButton*)button_close, GTK_RELIEF_NONE);
-    gtk_button_set_image((GtkButton*)button_close,
-                         gtk_image_new_from_stock(GTK_STOCK_CANCEL,
-                                                  GTK_ICON_SIZE_MENU));
-    
-    gtk_widget_set_tooltip_text(button_close, _("Discard changes"));
-    g_signal_connect_swapped((GObject*)button_close, "clicked",
-                             (GCallback)gtk_widget_destroy, window);
-    
-    gtk_box_pack_end(GTK_BOX(hbox), button_close, FALSE, FALSE, 0);
-    
-    /* Position window near status icon */
-    if (!prefs.no_icon)
+    /* Run the dialog */
+    gtk_widget_show_all(dialog);
+    if (gtk_dialog_run((GtkDialog*)dialog) == GTK_RESPONSE_ACCEPT)
     {
-      GdkRectangle area;
-      gtk_status_icon_get_geometry(status_icon, NULL, &area, NULL);
-      gtk_window_move((GtkWindow*)window, area.x, area.y);
+      /* Save changes done to the clipboard */
+      GtkTextIter start, end;
+      gtk_text_buffer_get_start_iter(clipboard_buffer, &start);
+      gtk_text_buffer_get_end_iter(clipboard_buffer, &end);
+      gchar* new_clipboard_text = gtk_text_buffer_get_text(clipboard_buffer, &start, &end, TRUE);
+      gtk_clipboard_set_text(clipboard, new_clipboard_text, -1);
+      g_free(new_clipboard_text);
     }
-    gtk_widget_show_all(window);
+    gtk_widget_destroy(dialog);
+    g_free(current_clipboard_text);
   }
 }
 
@@ -406,6 +348,7 @@ show_about_dialog(GtkMenuItem *menu_item, gpointer user_data)
                                              "Németh Tamás <ntomasz@vipmail.hu>\n"
                                              "Hasan Yılmaz <iletisim@hasanyilmaz.net>\n"
                                              "Jiro Kawada <jiro.kawada@gmail.com>\n"
+                                             "Phantom X <megaphantomx@bol.com.br>\n"
                                              "Gilberto \"Xyhthyx\" Miralla <xyhthyx@gmail.com>");
     
     gtk_about_dialog_set_license((GtkAboutDialog*)about_dialog, license);
@@ -694,18 +637,21 @@ static void
 status_icon_clicked(GtkStatusIcon *status_icon, gpointer user_data)
 {
   /* Check what type of click was recieved */
-  GdkEvent* current_event = gtk_get_current_event();
+  GdkModifierType state;
+  gtk_get_current_event_state(&state);
   /* Control click */
-  if (current_event->button.state == (GDK_MOD2_MASK | GDK_CONTROL_MASK))
+  if (state == GDK_MOD2_MASK+GDK_CONTROL_MASK || state == GDK_CONTROL_MASK)
   {
-    if (!actions_lock)
+    if (actions_lock == FALSE)
+    {
       g_timeout_add(POPUP_DELAY, show_actions_menu, NULL);
+    }
   }
   /* Normal click */
   else
+  {
     g_timeout_add(POPUP_DELAY, show_history_menu, NULL);
-  /* Free the event */
-  gdk_event_free(current_event);
+  }
 }
 
 /* Called when history global hotkey is pressed */
@@ -801,11 +747,11 @@ main(int argc, char *argv[])
       if (piped_string->len > 0)
       {
         /* Truncate new line character */
-        g_string_truncate(piped_string, (piped_string->len - 1));
+        /* g_string_truncate(piped_string, (piped_string->len - 1)); */
         /* Copy to clipboard */
-        GtkClipboard* cb = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
-        gtk_clipboard_set_text(cb, piped_string->str, -1);
-        gtk_clipboard_store(cb);
+        GtkClipboard* clip = gtk_clipboard_get(GDK_SELECTION_CLIPBOARD);
+        gtk_clipboard_set_text(clip, piped_string->str, -1);
+        gtk_clipboard_store(clip);
         /* Exit */
         return 0;
       }
@@ -830,6 +776,7 @@ main(int argc, char *argv[])
   g_slist_free(history);
   g_free(primary_text);
   g_free(clipboard_text);
+  g_free(synchronized_text);
   
   /* Exit */
   return 0;
